require File.dirname(__FILE__) + '/../spec_helper'


describe WikiController, "#route_for" do
  it do
    route_for(:controller => "wiki", :action => "show", :id => "123", :name => "www.my späce.de").should == "/wiki/123/show/www.my%20sp%C3%A4ce.de"
  end
end


describe "All wiki requests", :shared => true do
  before(:each) do
    controller.stub!(:load_object).and_return(true)
    controller.instance_variable_set("@obj", @obj = mock_model(Wiki, :to_param => "123"))

    controller.stub!(:ensure_object_is_active).and_return(true)
    controller.stub!(:ensure_object_is_permitted).and_return(true)
    controller.stub!(:enable_gui_to_update_its_menu_selection_in_preview_mode).and_return(true)
    controller.stub!(:set_google_expire_header)
  end
end


describe "All wiki page requests", :shared => true do
  it_should_behave_like 'All wiki requests'

  before(:each) do
    @page = mock_model(WikiPage)
    @obj.stub!(:find_or_build_page).and_return(@page)
  end

  it "should load the requested wiki page MyPage" do
    @obj.should_receive(:find_or_build_page).with("MyPage").and_return(@page)
    do_request
  end

  it "should assign the wiki page for the view" do
    do_request
    assigns[:page].should == @page
  end
end


describe "All wiki start page requests", :shared => true do
  it_should_behave_like 'All wiki requests'

  before(:each) do
    @obj.stub!(:find_or_create_start_page).and_return(@page)
  end

  it "should load the wiki page StartPage" do
    @obj.should_receive(:find_or_create_start_page).and_return(@page)
    do_request
  end

  it "should assign the wiki page for the view" do
    do_request
    assigns[:page].should == @page
  end

  it "should display the index template" do
    do_request
    response.should render_template(:index)
  end
end


describe WikiController, "GET /123" do
  it_should_behave_like 'All wiki start page requests'

  def do_request
    get 'index', :id => "123"
  end
end


describe WikiController, "GET /123/list" do
  it_should_behave_like 'All wiki requests'

  before(:each) do
    @pages = [mock("an array of wiki pages")]
    @obj.stub!(:find_pages).and_return(@pages)
  end

  def do_request
    get 'list', :id => "123"
  end

  it "should assign the array of wiki pages for the view" do
    do_request
    assigns[:pages].should == @pages
  end

  it "should load all pages for the current wiki" do
    @obj.should_receive(:find_pages).and_return(@pages)
    do_request
  end

  it "should display the list template" do
    do_request
    response.should render_template(:list)
  end
end


describe WikiController, "GET /123/show/MyPage" do
  it_should_behave_like 'All wiki page requests'

  def do_request
    get 'show', :id => "123", :name => 'MyPage'
  end

  it "should render the show template" do
    do_request
    response.should render_template(:show)
  end
end


describe WikiController, "GET /123/show/StartPage" do
  it_should_behave_like 'All wiki start page requests'

  def do_request
    get 'show', :id => "123", :name => 'StartPage'
  end
end


describe WikiController, "GET /123/edit/MyPage" do
  it_should_behave_like 'All wiki page requests'

  def do_request
    get 'edit', :id => "123", :name => "MyPage"
  end

  it "should display the edit template" do
    do_request
    response.should render_template(:edit)
  end
end


describe WikiController, "POST /123/edit/MyPage" do
  it_should_behave_like 'All wiki page requests'

  before(:each) do
    @page.stub!(:update_attributes)
  end

  def do_request
    post 'edit', :id => "123", :name => "MyPage", :page => {:body => "My new content"}
  end

  it "should update the page MyPage with the new content" do
    @page.should_receive(:update_attributes).with("body" => "My new content")
    do_request
  end

  it "should redirect to the show page" do
    do_request
    response.should redirect_to(:action => :show, :id => "123", :name => "MyPage")
  end
end


describe WikiController, "AJAX /123/delete/MyPage" do
  it_should_behave_like 'All wiki requests'

  before do
    @obj.stub!(:delete_page)
  end

  def do_request
    xhr :post, :delete, :id => 123, :name => "MyPage"
  end

  it "should delete the page" do
    @obj.should_receive(:delete_page).with("MyPage")
    do_request
  end

  it "should redirect to the start page" do
    do_request
    response.body.should satisfy { |b|
      # The used redirect calls javascript_for_obj which uses to_json if available and inspect
      # otherwise. Therefore we need to test both to be independent from the availability of json
      # which is currently loaded by the content model.
      # The first matches without and the second matches with json.
      b =~ %r|"http://test.host/wiki/123"| || b =~ %r|"http:\\/\\/test.host\\/wiki\\/123"|
    }
  end
end

