require File.dirname(__FILE__) + '/../spec_helper'

describe WikiHelper, "format_wiki_text" do
  it "should convert wiki links to markdown and process the result with markdown afterwards" do
    helper.should_receive(:convert_wiki_links).with("input").and_return("markdown")
    helper.should_receive(:markdown).with("markdown").and_return("html")
    helper.format_wiki_text("input").should == "html"
  end
end


describe WikiHelper, "convert_wiki_links" do
  use_private_method WikiHelper, :convert_wiki_links

  it "should convert wiki links to markdown links" do
    helper.should_receive(:generate_markdown_link).with("MyPage", nil).and_return("markdownlink")
    helper.convert_wiki_links("Text [[MyPage]] Text").should == "Text markdownlink Text"
  end

  it "should convert wiki links with title to markdown links with title" do
    helper.should_receive(:generate_markdown_link).with("MyPage", "Der Titel").and_return("markdownlink")
    helper.convert_wiki_links("Text [[MyPage][Der Titel]] Text").should == "Text markdownlink Text"
  end

  it "should recognize wiki links with special characters" do
    helper.should_receive(:generate_markdown_link).with("中国", nil).and_return("china")
    helper.should_receive(:generate_markdown_link).with("http://www.my späce.de", "Der Tütel").and_return("leerzeichen")
    helper.convert_wiki_links("Text [[中国]] [[http://www.my späce.de][Der Tütel]] Text").should == "Text china leerzeichen Text"
  end

  it "should convert every occurrence of wiki links" do
    helper.should_receive(:generate_markdown_link).exactly(4).times.and_return("foo")
    helper.convert_wiki_links("[[LinkZiel]] [[MyPage]] [[LinkZiel][Der Titel]] [[MyPage][Der Titel]]")
  end

  it "should leave the text unchanged if it contains no wiki links" do
    helper.convert_wiki_links("Kein LinkZiel ohne Klammern").should == "Kein LinkZiel ohne Klammern"
  end

  it "should return nil if the text is nil" do
    helper.convert_wiki_links(nil).should be_nil
  end
end


describe WikiHelper, "generate_markdown_link" do
  use_private_method WikiHelper, :generate_markdown_link
  use_private_method WikiHelper, :escape_markdown

  before do
    @wiki_pages = mock_model(WikiPage, :find_by_name => mock("a page"))
    assigns[:obj] = mock_model(Obj, :wiki_pages => @wiki_pages)
    helper.stub!(:generate_wiki_href).and_return('link')
  end

  it "should generate markdown links with a linked page name" do
    helper.should_receive(:generate_wiki_href).with("MyPage").and_return("expectedhref")
    helper.generate_markdown_link("MyPage", nil).should == "[MyPage](expectedhref)"
  end

  it "should generate markdown links with a linked title" do
    helper.should_receive(:generate_wiki_href).with("MyPage").and_return("expectedhref")
    helper.generate_markdown_link("MyPage", "Der Titel").should == "[Der Titel](expectedhref)"
  end

  it "should escape markdown reserved characters in the linked text but not in the URL" do
    helper.stub!(:generate_wiki_href).and_return('\ ` * _ {} [] () # + - . !')
    helper.should_receive(:escape_markdown).with('\ ` * _ {} [] () # + - . !').and_return('escapedMarkdown')
    helper.generate_markdown_link('\ ` * _ {} [] () # + - . !', nil).should == '[escapedMarkdown](\ ` * _ {} [] () # + - . !)'
  end

  it "should style links to non existing wiki pages" do
    @wiki_pages.should_receive(:find_by_name).with('nonexisting').and_return(nil)
    helper.generate_markdown_link('nonexisting', nil).should == "[nonexisting](link){: .wiki_nonexisting_page}"
  end

  it "should not style links to existing wiki pages" do
    @wiki_pages.should_receive(:find_by_name).with('existing').and_return(mock('existing'))
    helper.generate_markdown_link('existing', nil).should == "[existing](link)"
  end
end


describe WikiHelper, "generate_wiki_href" do
  use_private_method WikiHelper, :generate_wiki_href

  it "should generate wiki URLs for wiki pages" do
    assigns[:obj] = mock_model(Obj, :to_param => "1234")
    helper.should_receive(:wiki_path).with(:id => assigns[:obj], :action => "show", :name => "MyPage").and_return("the_wiki_path")
    helper.generate_wiki_href("MyPage").should == "the_wiki_path"
  end
end


describe WikiHelper, "page_title" do
  it "should display the editable object's title for the StartPage" do
    assigns[:obj] = mock_model(Obj)
    assigns[:page] = mock_model(WikiPage, :start_page? => true)
    helper.should_receive(:display).with(assigns[:obj], :title).and_return("editable_title")
    helper.page_title.should == "editable_title"
  end

  it "should display the name of the wiki page otherwise" do
    assigns[:page] = mock_model(WikiPage, :name => "page's name", :start_page? => false)
    helper.page_title.should == "page's name"
  end

  it "should display the editable object's title if there is no page (i.e. for list)" do
    assigns[:obj] = mock_model(Obj)
    assigns[:page] = nil
    helper.should_receive(:display).with(assigns[:obj], :title).and_return("editable_title")
    helper.page_title.should == "editable_title"
  end
end


describe WikiHelper, "escape_markdown" do
  use_private_method WikiHelper, :escape_markdown

  it "should escape all markdown reserved characters" do
    helper.escape_markdown('\ ` * _ {} [] () # + - . ! \ ` * _ {} [] () # + - . !').should ==
        "\\\\ \\` \\* \\_ \\{\\} \\[\\] \\(\\) \\# \\+ \\- \\. \\! \\\\ \\` \\* \\_ \\{\\} \\[\\] \\(\\) \\# \\+ \\- \\. \\!"
  end
end
